@@ -21,7 +21,6 @@
 
 #include <config-ktorrent.h>
 
-#include <kdebug.h>
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -29,8 +28,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <qfile.h>
-#include <kio/netaccess.h>
-#include <klocale.h>
+#include <klocalizedstring.h>
 #include <kfileitem.h>
 #include <util/array.h>
 #include <util/fileops.h>
@@ -78,7 +76,7 @@ namespace bt
 	{
 		// by default always try read write
 		fptr = new QFile(path);
-		connect(fptr,SIGNAL(aboutToClose()),this,SLOT(aboutToClose()));
+		connect(fptr, &QFile::aboutToClose, this, &CacheFile::aboutToClose);
 		bool ok = false;
 		if (!(ok = fptr->open(QIODevice::ReadWrite)))
 		{
@@ -454,7 +452,7 @@ namespace bt
 	
 	
 		
-	void CacheFile::preallocate(PreallocationThread* /*prealloc*/)
+	void CacheFile::preallocate(PreallocationThread* prealloc)
 	{
 		QMutexLocker lock(&mutex);
 		
@@ -504,6 +502,7 @@ namespace bt
 		}
 
 		file_size = FileSize(fd);
+		prealloc->written(file_size);
 		Out(SYS_GEN|LOG_DEBUG) << "file_size = " << file_size << endl;
 		if (close_again)
 			closeTemporary();
