@@ -21,63 +21,63 @@
 
 #include <QFile>
 #include <QTextStream>
-#include <util/file.h>
-#include <util/log.h>
-#include <diskio/chunkmanager.h>
-#include <diskio/piecedata.h>
+#include <klocalizedstring.h>
+
 #include <torrent/torrent.h>
 #include <peer/peermanager.h>
+#include <peer/peer.h>
+#include <diskio/chunkmanager.h>
+#include <diskio/piecedata.h>
+#include <util/file.h>
+#include <util/log.h>
 #include <util/error.h>
-#include "chunkdownload.h"
 #include <util/sha1hash.h>
 #include <util/array.h>
-#include <peer/peer.h>
+#include "chunkdownload.h"
 #include <download/piece.h>
 #include <peer/peerdownloader.h>
 #include <peer/badpeerslist.h>
 #include <util/functions.h>
 #include <interfaces/monitorinterface.h>
-#include <peer/packetwriter.h>
 #include <peer/accessmanager.h>
+#include <peer/chunkcounter.h>
 #include "chunkselector.h"
 #include "version.h"
 #include "webseed.h"
 
 
+
 namespace bt
 {
 
 	bool Downloader::use_webseeds = true;
 	
 	Downloader::Downloader(Torrent & tor,PeerManager & pman,ChunkManager & cman) 
-	: tor(tor),pman(pman),cman(cman),downloaded(0),tmon(0),chunk_selector(0),webseed_endgame_mode(false)
+	: tor(tor),pman(pman),cman(cman),bytes_downloaded(0),tmon(0),chunk_selector(0),webseed_endgame_mode(false)
 	{
 		webseeds_on = use_webseeds;
 		pman.setPieceHandler(this);
-		chunk_selector = new ChunkSelector(cman,*this,pman);
+		chunk_selector = new ChunkSelector();
+		chunk_selector->init(&cman,this,&pman);
 		
 		Uint64 total = tor.getTotalSize();
-		downloaded = (total - cman.bytesLeft());
+		bytes_downloaded = (total - cman.bytesLeft());
 		curr_chunks_downloaded = 0;
 		unnecessary_data = 0;
 	
 		current_chunks.setAutoDelete(true);
-		connect(&pman,SIGNAL(newPeer(Peer* )),this,SLOT(onNewPeer(Peer* )));
-		connect(&pman,SIGNAL(peerKilled(Peer* )),this,SLOT(onPeerKilled(Peer*)));
 		
 		active_webseed_downloads = 0;
-		const KUrl::List & urls = tor.getWebSeeds();
-		foreach (const KUrl &u,urls)
+		const QList<QUrl> & urls = tor.getWebSeeds();
+		foreach (const QUrl &u,urls)
 		{
-			if (u.protocol() == "http")
+			if (u.scheme() == QLatin1String("http"))
 			{
 				WebSeed* ws = new WebSeed(u,false,tor,cman);
 				webseeds.append(ws);
-				connect(ws,SIGNAL(chunkReady(Chunk*)),this,SLOT(onChunkReady(Chunk*)));
-				connect(ws,SIGNAL(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)),
-						this,SLOT(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)));
-				connect(ws,SIGNAL(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)),
-						this,SLOT(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)));
+				connect(ws, &WebSeed::chunkReady, this, &Downloader::onChunkReady);
+				connect(ws, &WebSeed::chunkDownloadStarted, this, &Downloader::chunkDownloadStarted);
+				connect(ws, &WebSeed::chunkDownloadFinished, this, &Downloader::chunkDownloadFinished);
 			}
 		}
 		
@@ -106,13 +106,14 @@ namespace bt
 	
 	void Downloader::setChunkSelector(ChunkSelectorInterface* csel)
 	{
-		if (chunk_selector)
-			delete chunk_selector;
+		delete chunk_selector;
 		
 		if (!csel) // check if a custom one was provided, if not create a default one
-			chunk_selector = new ChunkSelector(cman,*this,pman);
+			chunk_selector = new ChunkSelector();
 		else
 			chunk_selector = csel;
+		
+		chunk_selector->init(&cman,this,&pman);
 	}
 
 	
@@ -137,15 +138,15 @@ namespace bt
 				tmon->downloadRemoved(cd);
 				
 			if (ok)
-				downloaded += p.getLength();
+				bytes_downloaded += p.getLength();
 			
 			if (!finished(cd))
 			{
 				// if the chunk fails don't count the bytes downloaded
-				if (cd->getChunk()->getSize() > downloaded)
-					downloaded = 0;
+				if (cd->getChunk()->getSize() > bytes_downloaded)
+					bytes_downloaded = 0;
 				else
-					downloaded -= cd->getChunk()->getSize();
+					bytes_downloaded -= cd->getChunk()->getSize();
 				current_chunks.erase(p.getIndex());
 			}
 			else
@@ -161,7 +162,7 @@ namespace bt
 		else
 		{
 			if (ok)
-				downloaded += p.getLength();
+				bytes_downloaded += p.getLength();
 		}
 			
 		if (!ok)
@@ -384,7 +385,7 @@ namespace bt
 	}
 	
 
-	bool Downloader::areWeDownloading(Uint32 chunk) const
+	bool Downloader::downloading(Uint32 chunk) const
 	{
 		return current_chunks.find(chunk) != 0;
 	}
@@ -400,7 +401,7 @@ namespace bt
 				return false;
 		}
 		
-		return !areWeDownloading(chunk);
+		return !downloading(chunk);
 	}
 	
 	Uint32 Downloader::numDownloadersForChunk(Uint32 chunk) const
@@ -412,24 +413,19 @@ namespace bt
 		return cd->getNumDownloaders();
 	}
 	
-	void Downloader::onNewPeer(Peer* peer)
-	{		
-		PieceDownloader* pd = peer->getPeerDownloader();
-		piece_downloaders.append(pd);
+	void Downloader::addPieceDownloader(PieceDownloader* peer)
+	{
+		piece_downloaders.append(peer);
 	}
 
-	void Downloader::onPeerKilled(Peer* peer)
+	void Downloader::removePieceDownloader(PieceDownloader* peer)
 	{
-		PieceDownloader* pd = peer->getPeerDownloader();
-		if (pd)
+		for (CurChunkItr i = current_chunks.begin();i != current_chunks.end();++i)
 		{
-			for (CurChunkItr i = current_chunks.begin();i != current_chunks.end();++i)
-			{
-				ChunkDownload* cd = i->second;
-				cd->killed(pd);
-			}
-			piece_downloaders.removeAll(pd);
+			ChunkDownload* cd = i->second;
+			cd->killed(peer);
 		}
+		piece_downloaders.removeAll(peer);
 	}
 	
 	bool Downloader::finished(ChunkDownload* cd)
@@ -452,12 +448,7 @@ namespace bt
 				
 				cman.chunkDownloaded(c->getIndex());
 				Out(SYS_GEN|LOG_IMPORTANT) << "Chunk " << c->getIndex() << " downloaded " << endl;
-				// tell everybody we have the Chunk
-				for (Uint32 i = 0;i < pman.getNumConnectedPeers();i++)
-				{
-					pman.getPeer(i)->getPacketWriter().sendHave(c->getIndex());
-				}
-				
+				pman.sendHave(c->getIndex());
 				emit chunkDownloaded(c->getIndex());
 			}
 			catch (Error & e)
@@ -482,7 +473,7 @@ namespace bt
 			PieceDownloader* only = cd->getOnlyDownloader();
 			if (only)
 			{
-				Peer* p = pman.findPeer(only);
+				Peer::Ptr p = pman.findPeer(only);
 				if (!p)
 					return false;
 				
@@ -605,7 +596,7 @@ namespace bt
 			return;
 
 		// recalculate downloaded bytes
-		downloaded = (tor.getTotalSize() - cman.bytesLeft());
+		bytes_downloaded = (tor.getTotalSize() - cman.bytesLeft());
 
 		CurrentChunksHeader chdr;
 		fptr.read(&chdr,sizeof(CurrentChunksHeader));
@@ -653,7 +644,7 @@ namespace bt
 			else
 			{
 				current_chunks.insert(hdr.index,cd);
-				downloaded += cd->bytesDownloaded();
+				bytes_downloaded += cd->bytesDownloaded();
 		
 				if (tmon)
 					tmon->downloadStarted(cd);
@@ -739,9 +730,9 @@ namespace bt
 		chunk_selector->reinsert(chunk);
 	}
 	
-	void Downloader::dataChecked(const BitSet & ok_chunks)
+	void Downloader::dataChecked(const bt::BitSet& ok_chunks, Uint32 from, Uint32 to)
 	{
-		for (Uint32 i = 0;i < ok_chunks.getNumBits();i++)
+		for (Uint32 i = from;i < ok_chunks.getNumBits() && i <= to;i++)
 		{
 			ChunkDownload* cd = current_chunks.find(i);
 			if (ok_chunks.get(i) && cd)
@@ -754,38 +745,26 @@ namespace bt
 				current_chunks.erase(i);
 			}
 		}
-		chunk_selector->dataChecked(ok_chunks);
+		chunk_selector->dataChecked(ok_chunks, from, to);
 	}
 	
 	void Downloader::recalcDownloaded()
 	{
 		Uint64 total = tor.getTotalSize();
-		downloaded = (total - cman.bytesLeft());
+		bytes_downloaded = (total - cman.bytesLeft());
 	}
 	
 	void Downloader::onChunkReady(Chunk* c)
 	{
-		PieceDataPtr piece = c->getPiece(0,c->getSize(),false);
-		
+		WebSeed* ws = webseeds_chunks.find(c->getIndex());
 		webseeds_chunks.erase(c->getIndex());
-		if (!piece)
-		{
-			// reset chunk but only when no other peer is downloading it
-			if (!current_chunks.find(c->getIndex()))
-				cman.resetChunk(c->getIndex());
-			
-			chunk_selector->reinsert(c->getIndex());
-			return;
-		}
-		
-
-		SHA1Hash h = SHA1Hash::generate(piece->data(),c->getSize());
-		if (tor.verifyHash(h,c->getIndex()))
+		PieceData::Ptr piece = c->getPiece(0,c->getSize(),true);
+		if (piece && c->checkHash(tor.getHash(c->getIndex())))
 		{
 			// hash ok so save it
 			try
 			{
-				downloaded += c->getSize();
+				bytes_downloaded += c->getSize();
 				
 				foreach (WebSeed* ws,webseeds)
 				{
@@ -809,10 +788,7 @@ namespace bt
 			
 				Out(SYS_GEN|LOG_IMPORTANT) << "Chunk " << c->getIndex() << " downloaded via webseed ! " << endl;
 				// tell everybody we have the Chunk
-				for (Uint32 i = 0;i < pman.getNumConnectedPeers();i++)
-				{
-					pman.getPeer(i)->getPacketWriter().sendHave(c->getIndex());
-				}
+				pman.sendHave(c->getIndex());
 			}
 			catch (Error & e)
 			{
@@ -823,14 +799,12 @@ namespace bt
 		else
 		{
 			Out(SYS_GEN|LOG_IMPORTANT) << "Hash verification error on chunk "  << c->getIndex() << endl;
-			Out(SYS_GEN|LOG_IMPORTANT) << "Is        : " << h << endl;
-			Out(SYS_GEN|LOG_IMPORTANT) << "Should be : " << tor.getHash(c->getIndex()) << endl;
-		
 			// reset chunk but only when no other peer is downloading it
 			if (!current_chunks.find(c->getIndex()))
 				cman.resetChunk(c->getIndex());
 			
 			chunk_selector->reinsert(c->getIndex());
+			ws->disable(i18n("Disabled because webseed does not match torrent"));
 		}
 	}
 	
@@ -852,7 +826,7 @@ namespace bt
 			tmon->downloadRemoved(cd);
 	}
 	
-	WebSeed* Downloader::addWebSeed(const KUrl & url)
+	WebSeed* Downloader::addWebSeed(const QUrl &url)
 	{
 		// Check for dupes
 		foreach (WebSeed* ws,webseeds)
@@ -863,15 +837,13 @@ namespace bt
 		
 		WebSeed* ws = new WebSeed(url,true,tor,cman);
 		webseeds.append(ws);
-		connect(ws,SIGNAL(chunkReady(Chunk*)),this,SLOT(onChunkReady(Chunk*)));
-		connect(ws,SIGNAL(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)),
-				this,SLOT(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)));
-		connect(ws,SIGNAL(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)),
-				this,SLOT(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)));
+		connect(ws, &WebSeed::chunkReady, this, &Downloader::onChunkReady);
+		connect(ws, &WebSeed::chunkDownloadStarted, this, &Downloader::chunkDownloadStarted);
+		connect(ws, &WebSeed::chunkDownloadFinished, this, &Downloader::chunkDownloadFinished);
 		return ws;
 	}
 		
-	bool Downloader::removeWebSeed(const KUrl & url)
+	bool Downloader::removeWebSeed(const QUrl &url)
 	{
 		foreach (WebSeed* ws,webseeds)
 		{
@@ -912,13 +884,13 @@ namespace bt
 		foreach (WebSeed* ws,webseeds)
 		{
 			if (ws->isUserCreated())
-				out << ws->getUrl().prettyUrl() << ::endl;
+				out << ws->getUrl().toDisplayString() << ::endl;
 		}
 		out << "====disabled====" << ::endl;
 		foreach (WebSeed* ws,webseeds)
 		{
 			if (!ws->isEnabled())
-				out << ws->getUrl().prettyUrl() << ::endl;
+				out << ws->getUrl().toDisplayString() << ::endl;
 		}
 	}
 	
@@ -936,14 +908,14 @@ namespace bt
 		while (!in.atEnd())
 		{
 			QString line = in.readLine();
-			if (line == "====disabled====")
+			if (line == QLatin1String("====disabled===="))
 			{
 				disabled_list_found = true;
 				continue;
 			}
 			
-			KUrl url(line);
-			if (!url.isValid() || url.protocol() != "http")
+			QUrl url(line);
+			if (!url.isValid() || url.scheme() != QLatin1String("http"))
 				continue;
 				
 			if (disabled_list_found)
@@ -961,11 +933,9 @@ namespace bt
 			{
 				WebSeed* ws = new WebSeed(url,true,tor,cman);
 				webseeds.append(ws);
-				connect(ws,SIGNAL(chunkReady(Chunk*)),this,SLOT(onChunkReady(Chunk*)));
-				connect(ws,SIGNAL(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)),
-						this,SLOT(chunkDownloadStarted(WebSeedChunkDownload*,Uint32)));
-				connect(ws,SIGNAL(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)),
-						this,SLOT(chunkDownloadFinished(WebSeedChunkDownload*,Uint32)));
+				connect(ws, &WebSeed::chunkReady, this, &Downloader::onChunkReady);
+				connect(ws, &WebSeed::chunkDownloadStarted, this, &Downloader::chunkDownloadStarted);
+				connect(ws, &WebSeed::chunkDownloadFinished, this, &Downloader::chunkDownloadFinished);
 			}
 		}
 	}
@@ -978,7 +948,12 @@ namespace bt
 		}
 	}
 	
-	ChunkDownload* Downloader::getDownload(Uint32 chunk)
+	ChunkDownload* Downloader::download(Uint32 chunk)
+	{
+		return current_chunks.find(chunk);
+	}
+	
+	const bt::ChunkDownload* Downloader::download(Uint32 chunk) const
 	{
 		return current_chunks.find(chunk);
 	}
@@ -987,7 +962,4 @@ namespace bt
 	{
 		use_webseeds = on;
 	}
-
 }
-
-#include "downloader.moc"
