@@ -24,18 +24,20 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
-#include <klocale.h>
+#include <klocalizedstring.h>
 #include <kio/job.h> 
-#include <kio/netaccess.h>
 #include <kio/copyjob.h> 
 #include <solid/device.h>
 #include <solid/storageaccess.h>
+#include <solid/storagedrive.h>
+#include <solid/storagevolume.h>
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
-#include <qdir.h>
-#include <qfile.h>
-#include <qstringlist.h>
+#include <QDir>
+#include <QFile>
+#include <QStringList>
+#include <QSet>
 #include "error.h"
 #include "log.h"
 #include "file.h"
@@ -51,6 +53,10 @@
 #define NAME_MAX 255
 #endif
 
+#ifndef PATH_MAX
+#define PATH_MAX 4096
+#endif
+
 #ifdef HAVE_XFS_XFS_H
 
 #if !defined(HAVE___S64) || !defined(HAVE___U64)
@@ -73,12 +79,16 @@ typedef	int64_t		__s64;
 #endif
 
 #ifndef Q_WS_WIN
+# ifdef Q_OS_LINUX
+# include <mntent.h>
+# endif
 #include <sys/statvfs.h>
 #endif
 #ifdef CopyFile
 #undef CopyFile
 #endif
 
+
 namespace bt
 {
 	void MakeDir(const QString & dir,bool nothrow)
@@ -171,49 +181,52 @@ namespace bt
 	void Move(const QString & src,const QString & dst,bool nothrow,bool silent)
 	{
 		//	Out() << "Moving " << src << " -> " << dst << endl;
-		KIO::CopyJob *mv = KIO::move(KUrl(src),KUrl(dst),silent ? KIO::HideProgressInfo|KIO::Overwrite : KIO::DefaultFlags); 
-		if (!KIO::NetAccess::synchronousRun(mv , 0)) 
+		KIO::CopyJob *mv = KIO::move(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dst),
+                                             silent ? KIO::HideProgressInfo|KIO::Overwrite : KIO::DefaultFlags);
+		if (!mv->exec())
 		{
 			if (!nothrow)
 				throw Error(i18n("Cannot move %1 to %2: %3",
 							src,dst,
-							KIO::NetAccess::lastErrorString()));
+							mv->errorString()));
 			else
 				Out(SYS_DIO|LOG_NOTICE) << QString("Error : Cannot move %1 to %2: %3")
 					.arg(src).arg(dst)
-					.arg(KIO::NetAccess::lastErrorString()) << endl;
+					.arg(mv->errorString()) << endl;
 
 		}
 	}
 
 	void CopyFile(const QString & src,const QString & dst,bool nothrow)
 	{
-		if (!KIO::NetAccess::file_copy(KUrl(src),KUrl(dst)))
+		KIO::FileCopyJob* copy = KIO::file_copy(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dst));
+		if (!copy->exec())
 		{
 			if (!nothrow)
 				throw Error(i18n("Cannot copy %1 to %2: %3",
 							src,dst,
-							KIO::NetAccess::lastErrorString()));
+							copy->errorString()));
 			else
 				Out(SYS_DIO|LOG_NOTICE) << QString("Error : Cannot copy %1 to %2: %3")
 					.arg(src).arg(dst)
-					.arg(KIO::NetAccess::lastErrorString()) << endl;
+					.arg(copy->errorString()) << endl;
 
 		}
 	}
 
 	void CopyDir(const QString & src,const QString & dst,bool nothrow)
 	{
-		if (!KIO::NetAccess::dircopy(KUrl(src),KUrl(dst),0))
+		KIO::CopyJob* copy = KIO::copy(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dst));
+		if (!copy->exec())
 		{
 			if (!nothrow)
 				throw Error(i18n("Cannot copy %1 to %2: %3",
 							src,dst,
-							KIO::NetAccess::lastErrorString()));
+							copy->errorString()));
 			else
 				Out(SYS_DIO|LOG_NOTICE) << QString("Error : Cannot copy %1 to %2: %3")
 					.arg(src).arg(dst)
-					.arg(KIO::NetAccess::lastErrorString()) << endl;
+					.arg(copy->errorString()) << endl;
 
 		}
 	}
@@ -226,21 +239,21 @@ namespace bt
 	static bool DelDir(const QString & fn)
 	{
 		QDir d(fn);
-		QStringList subdirs = d.entryList(QDir::Dirs);
+		const QStringList subdirs = d.entryList(QDir::Dirs);
 
-		for (QStringList::iterator i = subdirs.begin(); i != subdirs.end();i++)
+		for (auto i = subdirs.begin(); i != subdirs.end();i++)
 		{
 			QString entry = *i;
 
-			if (entry == ".." || entry == ".")
+			if (entry == QLatin1String("..") || entry == QLatin1String("."))
 				continue;
 
 			if (!DelDir(d.absoluteFilePath(entry)))
 				return false;	
 		}
 
-		QStringList files = d.entryList(QDir::Files | QDir::System | QDir::Hidden);
-		for (QStringList::iterator i = files.begin(); i != files.end();i++)
+		const QStringList files = d.entryList(QDir::Files | QDir::System | QDir::Hidden);
+		for (auto i = files.begin(); i != files.end();i++)
 		{
 			QString file = d.absoluteFilePath(*i);
 			QFile fp(file);
@@ -463,7 +476,7 @@ namespace bt
 	bool FileNameToLong(const QString & path)
 	{
 		int length = 0;
-		QStringList names = path.split("/");
+		QStringList names = path.split('/');
 		foreach (const QString & s, names)
 		{
 			QByteArray encoded = QFile::encodeName(s);
@@ -472,7 +485,7 @@ namespace bt
 			length += encoded.length();
 		}
 		
-		length += path.count("/");
+		length += path.count('/');
 		return length >= PATH_MAX;
 	}
 	
@@ -495,16 +508,16 @@ namespace bt
 		
 		do
 		{
-			base = base.left(base.length() - 1);
+			base.chop(1);
 		}while (fixed_len + QFile::encodeName(base).length() > NAME_MAX - 4 && base.length() != 0);
 		
-		base += "... "; // add ... so that the user knows the name is shortened
+		base += QLatin1String("... "); // add ... so that the user knows the name is shortened
 		
 		QString ret = base;
 		if (extra_number > 0)
 			ret += QString::number(extra_number);
 		if (ext.length() > 0)
-			ret += "." + ext;
+			ret += '.' + ext;
 		return ret;
 	}
 	
@@ -518,7 +531,7 @@ namespace bt
 		QFileInfo fi(path);
 		QString ext = fi.suffix();
 		QString name = fi.completeBaseName();
-		QString fpath = fi.path() + "/";
+		QString fpath = fi.path() + '/';
 
                 // calculate the fixed length, 1 is for the . between filename and extension
 		int fixed_len = QFile::encodeName(fpath).length();
@@ -533,16 +546,16 @@ namespace bt
 
 		do
 		{
-			name = name.left(name.length() - 1);
+			name.chop(1);
 		}while (fixed_len + QFile::encodeName(name).length() > max_len - 4 && name.length() != 0);
 
-		name += "... "; // add ... so that the user knows the name is shortened
+		name += QLatin1String("... "); // add ... so that the user knows the name is shortened
 
 		QString ret = fpath + name;
 		if (extra_number > 0)
 			ret += QString::number(extra_number);
 		if (ext.length() > 0)
-			ret += "." + ext;
+			ret += '.' + ext;
 
 		return ret;
 	}
@@ -550,7 +563,7 @@ namespace bt
 	QString ShortenFileName(const QString & path,int extra_number)
 	{
 		QString assembled = "/";
-		QStringList names = path.split("/",QString::SkipEmptyParts);
+		QStringList names = path.split('/',QString::SkipEmptyParts);
 		int cnt = 0;
 		for (QStringList::iterator i = names.begin();i != names.end();i++)
 		{
@@ -618,23 +631,64 @@ namespace bt
 #endif
 		return ret;
 	}
-	
-	QString MountPoint(const QString& path)
+
+	QSet<QString> AccessibleMountPoints()
 	{
-		QList<Solid::Device> devs = Solid::Device::listFromType(Solid::DeviceInterface::StorageAccess);
-		QString mountpoint;
+        QSet<QString> result;
+#ifdef Q_OS_LINUX
+		FILE* fptr = setmntent("/proc/mounts", "r");
+		if(!fptr)
+			return result;
+
+		struct mntent mnt;
+		char buf[PATH_MAX];
+		while(getmntent_r(fptr, &mnt, buf, PATH_MAX))
+		{
+			result.insert(QString(mnt.mnt_dir));
+		}
 		
+		endmntent(fptr);
+		
+#else
+		QList<Solid::Device> devs = Solid::Device::listFromType(Solid::DeviceInterface::StorageAccess);
 		foreach (Solid::Device dev,devs)
 		{
 			Solid::StorageAccess* sa = dev.as<Solid::StorageAccess>();
-			if (path.startsWith(sa->filePath()))
+			if(!sa->filePath().isEmpty() && sa->isAccessible())
+				result.insert(sa->filePath());
+		}
+#endif
+		return result;
+	}
+	
+	QString MountPoint(const QString& path)
+	{
+		QSet<QString> mount_points = AccessibleMountPoints();
+		QString mount_point;
+		foreach (const QString & mp, mount_points)
+        {
+            if (path.startsWith(mp) && (mount_point.isEmpty() || mp.startsWith(mount_point)))
 			{
-				if (mountpoint.isEmpty() || sa->filePath().startsWith(mountpoint))
-					mountpoint = sa->filePath();
+				mount_point = mp;
 			}
 		}
 		
-		return mountpoint;
+		return mount_point;
 	}
 
+
+	bool IsMounted(const QString& mount_point)
+	{
+		return AccessibleMountPoints().contains(mount_point);
+	}
+
+    QByteArray LoadFile(const QString& path)
+    {
+        QFile fptr(path);
+        if (fptr.open(QIODevice::ReadOnly))
+            return fptr.readAll();
+        else
+            throw Error(i18n("Unable to open file %1: %2").arg(path).arg(fptr.errorString()));
+    }
+
 }
