@@ -20,6 +20,7 @@
 #include "functions.h"
 #include <qdir.h>
 #include <errno.h>
+#include <unistd.h>
 #include <sys/time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -27,14 +28,17 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
-
-#include <qdatetime.h>
+#include <gcrypt.h>
 #include <QNetworkInterface>
-#include <kio/netaccess.h>
-#include <klocale.h>
-#include <kmimetype.h>
-#include <kglobal.h>
+#include <QTime>
+
+#include <kformat.h>
+#include <klocalizedstring.h>
+
 #include <interfaces/torrentinterface.h>
+#include <util/signalcatcher.h>
+#include <QMimeDatabase>
+#include <QMimeType>
 #include "error.h"
 #include "log.h"
 
@@ -43,119 +47,16 @@ namespace bt
 
 	bool IsMultimediaFile(const QString & filename)
 	{
-		KMimeType::Ptr ptr = KMimeType::findByPath(filename);
-		QString name = ptr->name();
-		return name.startsWith("audio") || name.startsWith("video") || name == "application/ogg";
+		QMimeType ptr = QMimeDatabase().mimeTypeForFile(filename);
+		QString name = ptr.name();
+		return name.startsWith(QLatin1String("audio")) || name.startsWith(QLatin1String("video")) || name == QLatin1String("application/ogg");
 	}
 	
 	QString DirSeparator()
 	{
 		//QString tmp;
 		//tmp.append(QDir::separator());
-		return "/";
-	}
-
-	void WriteUint64(Uint8* buf,Uint32 off,Uint64 val)
-	{
-		buf[off + 0] = (Uint8) ((val & 0xFF00000000000000ULL) >> 56);
-		buf[off + 1] = (Uint8) ((val & 0x00FF000000000000ULL) >> 48);
-		buf[off + 2] = (Uint8) ((val & 0x0000FF0000000000ULL) >> 40);
-		buf[off + 3] = (Uint8) ((val & 0x000000FF00000000ULL) >> 32);
-		buf[off + 4] = (Uint8) ((val & 0x00000000FF000000ULL) >> 24);
-		buf[off + 5] = (Uint8) ((val & 0x0000000000FF0000ULL) >> 16);
-		buf[off + 6] = (Uint8) ((val & 0x000000000000FF00ULL) >> 8);
-		buf[off + 7] = (Uint8) ((val & 0x00000000000000FFULL) >> 0);
-	}
-	
-	Uint64 ReadUint64(const Uint8* buf,Uint64 off)
-	{
-		Uint64 tmp =
-				((Uint64)buf[off] << 56) |
-				((Uint64)buf[off+1] << 48) |
-				((Uint64)buf[off+2] << 40) |
-				((Uint64)buf[off+3] << 32) |
-				((Uint64)buf[off+4] << 24) |
-				((Uint64)buf[off+5] << 16) |
-				((Uint64)buf[off+6] << 8) |
-				((Uint64)buf[off+7] << 0);
-
-		return tmp;
-	}
-
-	void WriteUint32(Uint8* buf,Uint32 off,Uint32 val)
-	{
-		buf[off + 0] = (Uint8) ((val & 0xFF000000) >> 24);
-		buf[off + 1] = (Uint8) ((val & 0x00FF0000) >> 16);
-		buf[off + 2] = (Uint8) ((val & 0x0000FF00) >> 8);
-		buf[off + 3] = (Uint8) (val & 0x000000FF);
-	}
-	
-	Uint32 ReadUint32(const Uint8* buf,Uint32 off)
-	{
-		return (buf[off] << 24) | (buf[off+1] << 16) | (buf[off+2] << 8) | buf[off + 3];
-	}
-	
-	void WriteUint16(Uint8* buf,Uint32 off,Uint16 val)
-	{
-		buf[off + 0] = (Uint8) ((val & 0xFF00) >> 8);
-		buf[off + 1] = (Uint8) (val & 0x000FF);
-	}
-
-	Uint16 ReadUint16(const Uint8* buf,Uint32 off)
-	{
-		return (buf[off] << 8) | buf[off + 1];
-	}
-	
-	
-	void WriteInt64(Uint8* buf,Uint32 off,Int64 val)
-	{
-		buf[off + 0] = (Uint8) ((val & 0xFF00000000000000ULL) >> 56);
-		buf[off + 1] = (Uint8) ((val & 0x00FF000000000000ULL) >> 48);
-		buf[off + 2] = (Uint8) ((val & 0x0000FF0000000000ULL) >> 40);
-		buf[off + 3] = (Uint8) ((val & 0x000000FF00000000ULL) >> 32);
-		buf[off + 4] = (Uint8) ((val & 0x00000000FF000000ULL) >> 24);
-		buf[off + 5] = (Uint8) ((val & 0x0000000000FF0000ULL) >> 16);
-		buf[off + 6] = (Uint8) ((val & 0x000000000000FF00ULL) >> 8);
-		buf[off + 7] = (Uint8) ((val & 0x00000000000000FFULL) >> 0);
-	}
-	
-	Int64 ReadInt64(const Uint8* buf,Uint32 off)
-	{
-		Int64 tmp =
-				((Int64)buf[off] << 56) |
-				((Int64)buf[off+1] << 48) |
-				((Int64)buf[off+2] << 40) |
-				((Int64)buf[off+3] << 32) |
-				((Int64)buf[off+4] << 24) |
-				((Int64)buf[off+5] << 16) |
-				((Int64)buf[off+6] << 8) |
-				((Int64)buf[off+7] << 0);
-
-		return tmp;
-	}
-	
-	void WriteInt32(Uint8* buf,Uint32 off,Int32 val)
-	{
-		buf[off + 0] = (Uint8) ((val & 0xFF000000) >> 24);
-		buf[off + 1] = (Uint8) ((val & 0x00FF0000) >> 16);
-		buf[off + 2] = (Uint8) ((val & 0x0000FF00) >> 8);
-		buf[off + 3] = (Uint8) (val & 0x000000FF);
-	}
-	
-	Int32 ReadInt32(const Uint8* buf,Uint32 off)
-	{
-		return (Int32)(buf[off] << 24) | (buf[off+1] << 16) | (buf[off+2] << 8) | buf[off + 3];
-	}
-	
-	void WriteInt16(Uint8* buf,Uint32 off,Int16 val)
-	{
-		buf[off + 0] = (Uint8) ((val & 0xFF00) >> 8);
-		buf[off + 1] = (Uint8) (val & 0x000FF);
-	}
-	
-	Int16 ReadInt16(const Uint8* buf,Uint32 off)
-	{
-		return (Int16)(buf[off] << 8) | buf[off + 1];
+		return QStringLiteral("/");
 	}
 	
 	void UpdateCurrentTime()
@@ -175,9 +76,39 @@ namespace bt
 	
 	Uint32 MaxOpenFiles()
 	{
-		struct rlimit lim;
-		getrlimit(RLIMIT_NOFILE,&lim);
-		return lim.rlim_cur;
+		static Uint32 max_open = 0;
+		if (max_open == 0)
+		{
+			struct rlimit lim;
+			getrlimit(RLIMIT_NOFILE,&lim);
+			max_open = lim.rlim_cur;
+		}
+		
+		return max_open;
+	}
+	
+	Uint32 CurrentOpenFiles()
+	{
+		return 0;
+		/*
+		//return 0;
+#ifdef Q_OS_LINUX
+		QDir dir(QString("/proc/%1/fd").arg(getpid()));
+		return dir.count();
+#else
+		return 0;
+#endif
+		*/
+	}
+	
+	bool OpenFileAllowed()
+	{
+		const Uint32 headroom = 50;
+		Uint32 max_open = MaxOpenFiles();
+		if (max_open == 0)
+			return true;
+		else
+			return max_open - CurrentOpenFiles() > headroom;
 	}
 
 	bool MaximizeLimits()
@@ -248,31 +179,71 @@ namespace bt
 			return QString();
 		else
 			return addr_list.front().ip().toString();
-	}	
+	}
+	
+	QStringList NetworkInterfaceIPAddresses(const QString& iface)
+	{
+		QNetworkInterface ni = QNetworkInterface::interfaceFromName(iface);
+		if (!ni.isValid())
+			return QStringList();
+		
+		QStringList ips;
+		QList<QNetworkAddressEntry> addr_list = ni.addressEntries();
+		foreach (const QNetworkAddressEntry & entry,addr_list)
+		{
+			ips << entry.ip().toString();
+		}
+		
+		return ips;
+	}
+	
+	QString CurrentIPv6Address()
+	{
+		QNetworkInterface ni = QNetworkInterface::interfaceFromName(net_iface);
+		if (!ni.isValid())
+		{
+			QList<QHostAddress> addrs = QNetworkInterface::allAddresses();
+			foreach (const QHostAddress & addr, addrs)
+			{
+                if (addr.protocol() == QAbstractSocket::IPv6Protocol && addr != QHostAddress::LocalHostIPv6 && !addr.isInSubnet(QHostAddress("FE80::"), 64))
+					return addr.toString();
+			}
+		}
+		else
+		{
+			QList<QNetworkAddressEntry> addrs = ni.addressEntries();
+			foreach (const QNetworkAddressEntry & entry,addrs)
+			{
+				QHostAddress addr = entry.ip();
+				if (addr.protocol() == QAbstractSocket::IPv6Protocol && addr != QHostAddress::LocalHostIPv6 && !addr.isInSubnet(QHostAddress("FE80::"), 64))
+					return addr.toString();
+			}
+		}
+		
+		return QString();
+	}
 
 		
 	QString BytesToString(Uint64 bytes)
 	{
-		KLocale* loc = KGlobal::locale();
-		return loc->formatByteSize(bytes,2);
+		static KFormat format;
+		return format.formatByteSize(bytes,2);
 	}
 
 	QString BytesPerSecToString(double speed)
 	{
-		KLocale* loc = KGlobal::locale();
-		return i18n("%1/s",loc->formatByteSize(speed,2));
+		static KFormat format;
+		return i18n("%1/s",format.formatByteSize(speed,2));
 	}
 
 	QString DurationToString(Uint32 nsecs)
 	{
-		KLocale* loc = KGlobal::locale();
-		QTime t;
 		int ndays = nsecs / 86400;
-		t = t.addSecs(nsecs % 86400);
-		QString s = loc->formatTime(t,true,true);
-		if (ndays > 0)
-			s = i18np("1 day ","%1 days ",ndays) + s;
-
+		QTime t = QTime(0,0,0,0).addSecs(nsecs % 86400);
+		QString s;
+		if (ndays > 0)     s = i18np("1 day ","%1 days ",ndays);
+		else if (t.hour()) s = t.toString();
+		else               s = t.toString(QStringLiteral("mm:ss"));
 		return s;
 	}
 	
@@ -304,7 +275,7 @@ namespace bt
 	}
 
 #ifdef Q_WS_WIN
-	bool InitWindowsSocketsAPI()
+	static bool InitWindowsSocketsAPI()
 	{
 		static bool initialized = false;
 		if (initialized)
@@ -320,4 +291,40 @@ namespace bt
 		return true;
 	}
 #endif
+
+	static bool InitGCrypt()
+	{
+		static bool initialized = false;
+		if (initialized)
+			return true;
+		
+		// If already initialized, don't do anything
+		if (gcry_control(GCRYCTL_INITIALIZATION_FINISHED_P))
+		{
+			initialized = true;
+			return true;
+		}
+		
+		if (!gcry_check_version("1.4.5"))
+		{
+			Out(SYS_GEN|LOG_NOTICE) << "Failed to initialize libgcrypt" << endl;
+			return false;
+		}
+		/* Disable secure memory. */
+		gcry_control(GCRYCTL_DISABLE_SECMEM, 0);
+		gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
+		initialized = true;
+		return true;
+	}
+	
+	bool InitLibKTorrent()
+	{
+		MaximizeLimits();
+		bool ret = InitGCrypt();
+#ifdef Q_WS_WIN
+		ret = InitWindowsSocketsAPI() && ret;
+#endif
+		return ret;
+	}
+
 }
