@@ -31,7 +31,7 @@
 #include <util/sha1hash.h>
 #include <util/sha1hashgen.h>
 
-#define BYTES_TO_SEND 1024*1024
+#define BYTES_TO_SEND 100*1024*1024
 
 using namespace utp;
 using namespace bt;
@@ -53,7 +53,7 @@ static QByteArray Generate(int size)
 	}
 	return ba;
 }
-
+/*
 static void Dump(const bt::Uint8* pkt, bt::Uint32 size,const QString & file)
 {
 	QFile fptr(file);
@@ -78,14 +78,14 @@ static void Dump(const bt::Uint8* pkt, bt::Uint32 size,const QString & file)
 		out << ::endl << ::endl << ::endl;
 	}
 }
-
+*/
 
 class SendThread : public QThread
 {
 	Q_OBJECT
 public:
 	
-	SendThread(Connection* outgoing,QObject* parent = 0) : QThread(parent),outgoing(outgoing)
+	SendThread(Connection::Ptr outgoing, UTPServer & srv,QObject* parent = 0) : QThread(parent),outgoing(outgoing),srv(srv)
 	{}
 	
 	virtual void run()
@@ -94,23 +94,25 @@ public:
 		QByteArray data = Generate(step);
 		bt::SHA1HashGen hgen;
 		
-		int sent = 0;
+		bt::Int64 sent = 0;
 		int off = 0;
-		while (sent < BYTES_TO_SEND)
+		net::Poll poller;
+		while (sent < BYTES_TO_SEND && outgoing->connectionState() != CS_CLOSED)
 		{
 			int to_send = step - off;
 			int ret = outgoing->send((const bt::Uint8*)data.data() + off,to_send);
-			Out(SYS_UTP|LOG_DEBUG) << "Transmitted " << ret << endl;
 			if (ret > 0)
 			{
 				hgen.update((const bt::Uint8*)data.data() + off,ret);
 				sent += ret;
 				off += ret;
 				off = off % step;
+				//Out(SYS_UTP|LOG_DEBUG) << "Transmitted " << sent << endl;
 			}
 			else if (ret == 0)
 			{
-				msleep(50);
+				srv.preparePolling(&poller, net::Poll::OUTPUT, outgoing);
+				poller.poll(1000);
 			}
 			else
 			{
@@ -121,11 +123,13 @@ public:
 		sleep(2);
 		Out(SYS_UTP|LOG_DEBUG) << "Transmitted " << sent << endl;
 		outgoing->dumpStats();
+		outgoing->close();
 		sent_hash = hgen.get();
 	}
 	
-	Connection* outgoing;
+	Connection::Ptr outgoing;
 	bt::SHA1Hash sent_hash;
+	UTPServer & srv;
 };
 
 class TransmitTest : public QEventLoop
@@ -137,12 +141,20 @@ public:
 	}
 	
 public slots:
-	void accepted(Connection* conn)
+	void accepted()
 	{
-		incoming = conn;
+		incoming = srv.acceptedConnection().toStrongRef();
+		incoming->setBlocking(true);
 		exit();
 	}
 	
+	void startConnect()
+	{
+		net::Address addr("127.0.0.1",port);
+		outgoing = srv.connectTo(addr);
+		outgoing->setBlocking(true);
+	}
+	
 	void endEventLoop()
 	{
 		exit();
@@ -151,9 +163,8 @@ public slots:
 private slots:
 	void initTestCase()
 	{
-		bt::InitLog("transmittest.log",false,true);
+		bt::InitLog("transmittest.log",false,true,false);
 		
-		incoming = outgoing = 0;
 		port = 50000;
 		while (port < 60000)
 		{
@@ -174,13 +185,16 @@ private slots:
 	
 	void testConnect()
 	{
-		net::Address addr("127.0.0.1",port);
-		connect(&srv,SIGNAL(accepted(Connection*)),this,SLOT(accepted(Connection*)),Qt::QueuedConnection);
-		outgoing = srv.connectTo(addr);
-		QVERIFY(outgoing != 0);
+		connect(&srv,SIGNAL(accepted()),this,SLOT(accepted()),Qt::QueuedConnection);
+		QTimer::singleShot(0,this,SLOT(startConnect())); 
 		QTimer::singleShot(5000,this,SLOT(endEventLoop())); // use a 5 second timeout
 		exec();
-		QVERIFY(incoming != 0);
+		QVERIFY(outgoing);
+		QVERIFY(incoming);
+		QVERIFY(incoming->connectionState() == CS_CONNECTED);
+		if (outgoing->connectionState() != CS_CONNECTED)
+			QVERIFY(outgoing->waitUntilConnected());
+		QVERIFY(outgoing->connectionState() == CS_CONNECTED);
 	}
 	
 	
@@ -196,17 +210,17 @@ private slots:
 		
 		bt::SHA1HashGen hgen;
 		
-		SendThread st(outgoing);
+		SendThread st(outgoing, srv);
 		st.start(); // The thread will start sending a whole bunch of data
-		int received = 0;
-		int failures = 0;
-		while (received < BYTES_TO_SEND)
+		bt::Int64 received = 0;
+		//int failures = 0;
+		incoming->setBlocking(true);
+		while (received < BYTES_TO_SEND && incoming->connectionState() != CS_CLOSED)
 		{
 			bt::Uint32 ba = incoming->bytesAvailable();
-			Out(SYS_UTP|LOG_DEBUG) << "Available " << ba << endl;
 			if (ba > 0)
 			{
-				failures = 0;
+				//failures = 0;
 				QByteArray data(ba,0);
 				int to_read = ba;//;qMin<bt::Uint32>(1024,ba);
 				int ret = incoming->recv((bt::Uint8*)data.data(),to_read);
@@ -215,17 +229,12 @@ private slots:
 				{
 					hgen.update((bt::Uint8*)data.data(),ret);
 					received += ret;
+					//Out(SYS_UTP|LOG_DEBUG) << "Received " << received << endl;
 				}
 			}
-			else
+			else if (incoming->connectionState() != CS_CLOSED)
 			{
-				usleep(100000);
-				if (st.isFinished())
-					break;
-				
-				failures++;
-				if (failures > 22)
-					break;
+				incoming->waitForData(1000);
 			}
 		}
 		
@@ -240,19 +249,18 @@ private slots:
 		Out(SYS_UTP|LOG_DEBUG) << "Received data hash: " << rhash.toString() << endl;
 		Out(SYS_UTP|LOG_DEBUG) << "Sent data hash:     " << st.sent_hash.toString() << endl;
 		QVERIFY(rhash == st.sent_hash);
-		
 	}
 	
 private:
 	
 	
 private:
-	Connection* incoming;
-	Connection* outgoing;
+	Connection::Ptr incoming;
+	Connection::Ptr outgoing;
 	utp::UTPServer srv;
 	int port;
 };
 
 QTEST_MAIN(TransmitTest)
 
-#include "transmittest.moc"
\ No newline at end of file
+#include "transmittest.moc"
